// 设置街景
function setStreetLayer() {
    displayStreetViewLayer = displayStreetViewLayer === false;
    try {
        if (displayStreetViewLayer) {
            console.log('进入街景模式');
            addStreetLayer()
        } else {
            console.log('退出街景模式');
            // 移除街景图层
            removeStreetLayer();
        }
    } catch (e) {
        alert(e)
    }
}

// 初始化街景图层，且不显示
function initStreetLayers(map, ip) {
    let tencentStreetLayer = new mars3d.layer.TencentLayer({
        "name": "TencentStreetViewLayer",
        "type": "tencent",
        "url": `http://${ip}/street/qq?s={s}&z={z}&x={x}&y={reverseY}`,
        "subdomains": "0123"
    });
    let googleStreetLayer = new mars3d.layer.GoogleLayer({
        "name": "GoogleStreetViewLayer",
        "type": "google",
        "crs": "gcj",
        "url": `http://${cdnConfig.cdnDomain}/cdn/sg/{z}/{x}/{y}.png`,
        "subdomains": ""
    });
    let baiduStreetLayer = new mars3d.layer.BaiduLayer({
        "name": "BaiduStreetViewLayer",
        "type": "baidu",
        "crs": "baidu",
        "url": "https://mapsv0.bdimg.com/tile/?udt=20200825&qt=tile&styles=pl&x={x}&y={y}&z={z}"
    });
//    map.addLayer(baiduStreetLayer, true);
    // map.addLayer(tencentStreetLayer, false);
//    map.addLayer(googleStreetLayer, true);
}


// 初始化空间
function initToolBar(map) {
    // 主页按钮
    let homeViewButton = new mars3d.control.ToolButton({
        title: '当前坐标',
        icon: 'i-tool-button-32 i-tool-button-home',
        click: (event) => {
            map.flyToPoint(myLocation, {radius: 1, maximumHeight: 7986317, duration: 3});
			if (isAndroid) {
				nativeinject.clickCurrent();
			}

        },
    });

    // 放大缩小控件
    let zoomViewButton = new mars3d.control.Zoom({
        zoomInClass: 'i-tool-button-32 i-tool-button-zoom-in',
        zoomOutClass: 'i-tool-button-32 i-tool-button-zoom-out',
    });

    // 添加街景控件
    let streetViewButton = new mars3d.control.ToolButton({
        title: '街景',
        icon: 'i-tool-button-64 i-tool-button-street-view-hidden',
        click: (event) => {
            setStreetLayer();
            // 切换图标
            if (displayStreetViewLayer === true) {
                let elements = document.getElementsByClassName('i-tool-button-street-view-hidden');
                if (elements.length > 0) {
                    elements[0].className = 'i-tool-button-64 i-tool-button-street-view-display';
                }
            } else {
                let elements = document.getElementsByClassName('i-tool-button-street-view-display');
                if (elements.length > 0) {
                    elements[0].className = 'i-tool-button-64 i-tool-button-street-view-hidden';
                }
            }
        },
    });

    // 添加标签控件
    let tagViewButton = new mars3d.control.ToolButton({
        title: '标签',
        icon: 'i-tool-button-32 i-tool-button-label-display',
        click: (event) => {
            // 切换图标
            if (displayTagLayer) {
                let elements = document.getElementsByClassName('i-tool-button-label-display');
                if (elements.length > 0) {
                    elements[0].className = 'i-tool-button-32 i-tool-button-label-hidden';
                }
            } else {
                let elements = document.getElementsByClassName('i-tool-button-label-hidden');
                if (elements.length > 0) {
                    elements[0].className = 'i-tool-button-32 i-tool-button-label-display';
                }
            }
            displayTagLayer = displayTagLayer === false;
            resetLayerDisplayStatus('tagsLayers', displayTagLayer);
        },
    });


    // 添加昼夜控件
    let lightingViewButton = new mars3d.control.ToolButton({
        title: '昼夜模式',
        icon: 'i-tool-button-32 i-tool-button-light',
        click: (event) => {
            map.scene.globe.enableLighting = map.scene.globe.enableLighting === false;
            map.shadows = map.shadows === false;
            // 切换图标
            if (!map.shadows) {
                let elements = document.getElementsByClassName('i-tool-button-light');
                if (elements.length > 0) {
                    elements[0].className = 'i-tool-button-32 i-tool-button-sun';
                }
            } else {
                let elements = document.getElementsByClassName('i-tool-button-sun');
                if (elements.length > 0) {
                    elements[0].className = 'i-tool-button-32 i-tool-button-light';
                }
            }
        },
    });
    map.addControl(homeViewButton);
    map.addControl(zoomViewButton);
    map.addControl(lightingViewButton);
//    map.addControl(tagViewButton);
    //map.addControl(streetViewButton);

}

// 初始化航天员图层
function initAstronautLayer(layer) {
    var graphic = new mars3d.graphic.DivBillboardEntity({
        position: [0, 90, 64.443],
        style: {
            html: `<div id="divpoint" class="divpoint-astronaut"></div>`,
            horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
            verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
            clampToGround: true,
        }
    });
    //layer.addGraphic(graphic);
}

// 初始化标签图层
function initTagsLayers() {
    let data = [
        {
            longitude: -74.04379350737892,
            latitude: 40.68874311623609,
            name: '纽约自由女神像',
            'gltf': './img/marker/ziyounvshen.gltf',
            heading: 90,
            pitch: 0,
            size: 200
        },
        {
            longitude: 103.85443576511817, latitude: 1.2868534620106291, name: '新加坡鱼尾狮', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
        {
            longitude: 4.897184786120059, latitude: 52.37389113197557, name: '荷兰阿姆斯特丹', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
        {
            longitude: 55.187181140967446, latitude: 25.139653812583067, name: '迪拜帆船酒店', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
        {
            longitude: 2.2954045679850132, latitude: 48.87370643003391, name: '法国凯旋门', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
        {
            longitude: 2.296717879159633,
            latitude: 48.85681702752232,
            name: '法国-巴黎-埃菲尔铁塔',
            'gltf': './img/marker/af.gltf',
            heading: 0,
            pitch: 90,
            size: 200
        },
        {
            longitude: -80.12757435930185, latitude: 25.784973573171627, name: '迈阿密海滩', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
        {
            longitude: 55.276356201531144, latitude: 25.19654046719174, name: '阿联酋哈利法塔', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
        {
            longitude: -118.26688280766905, latitude: 34.04190347748418, name: '洛杉矶斯台普斯湖人主场', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
        {
            longitude: 7.015988996335448, latitude: 43.55010960127415, name: '摩洛哥豪华游艇', 'gltf': '',
            heading: 0,
            pitch: 0,
            size: 100
        },
    ];
    var graphicLayer = new mars3d.layer.GraphicLayer({name: 'tagsLayers'});
    map.addLayer(graphicLayer);
    for (let i = 0; i < data.length; i++) {
        // addTagLayer(graphicLayer, data[i]);
        // addGraphicToLayer(graphicLayer, data[i]);
    }
    initAstronautLayer(graphicLayer)
}

function addGraphicToLayer(graphicLayer, data) {
    let gltf = data.gltf;
    if (!gltf) {
        // gltf = 'https://a.amap.com/jsapi_demos/static/gltf/Duck.gltf'
        return;
    }
    var graphic = new mars3d.graphic.ModelEntity({
        name: '风机',
        position: new mars3d.LatLngPoint(data.longitude, data.latitude, 10),
        style: {
            url: gltf,
            heading: data.heading,
            pitch: data.pitch,
            scale: 30,
            minimumPixelSize: data.size,
        },
    });
    // 添加点击监听事件
    graphic.on(mars3d.EventType.click, function (event) {
        set_cache(streetViewLocationCacheKey, JSON.stringify(data));
        console.log('before', data);
        map.flyToGraphic(graphic, {complete: openStreetView});
    });
    graphicLayer.addGraphic(graphic)
}

function openStreetView(e) {
    // 取出缓存中的坐标
    let data = get_cache(streetViewLocationCacheKey);
    console.log("openStreetView",data);
    if (isAndroid) {
        nativeinject.gotoStreetView(data);
    }
    if (isiOS) {
        window.webkit.messageHandlers.gotoStreetView.postMessage(data);
    }
    // 清除缓存
    clear_cache();
}

// 添加标签层
function addTagLayer(graphicLayer, data) {
    var graphic = new mars3d.graphic.DivBillboardEntity({
        position: new mars3d.LatLngPoint(data.longitude, data.latitude, 10),
        style: {
            html: `<div id="divpoint" class="divpoint-scenic"></div>`,
            horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
            verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
            clampToGround: true
        }
    });
    // 添加点击监听事件
    graphic.on(mars3d.EventType.click, function (event) {
        set_cache(streetViewLocationCacheKey, JSON.stringify(data));
        console.log('before', data);
        map.flyToGraphic(graphic, {complete: openStreetView});
    });

    // 添加标签到标签层
    graphicLayer.addGraphic(graphic);
}

// 设置图层显示/不显示
function resetLayerDisplayStatus(layerName, display) {
    let mapLayers = map.getLayers();
    for (let i = 0; i < mapLayers.length; i++) {
        let currentLayer = mapLayers[i];
        if (currentLayer.name === layerName) {
            mapLayers[i].show = display;
        }
    }
}

// 添加街景图层
function addStreetLayer() {
    // 显示街景图层
    resetLayerDisplayStatus('GoogleStreetViewLayer', true);
    // resetLayerDisplayStatus('TencentStreetViewLayer', true);
    resetLayerDisplayStatus('BaiduStreetViewLayer', true);
    // 绑定
    bindMouseClickEvent(map);
}

// 删除街景图层
function removeStreetLayer() {
    console.log("removeStreetLayer");
    // 设置街景不显示
    resetLayerDisplayStatus('GoogleStreetViewLayer', false);
    // resetLayerDisplayStatus('TencentStreetViewLayer', false);
    resetLayerDisplayStatus('BaiduStreetViewLayer', false);
    unbindClickEvent(map)
}

// 获取图层配置
function getLayerConfig(ip) {
    return {
        "layers": [
            {
                "name": "谷歌底图",
                "type": "xyz",
                "layer": "img_d",
                // "url": `http://${ip}/v2/satellite/google?x={x}&y={y}&z={z}&s={s}`,
                "url": `http://${cdnConfig.cdnDomain}/cdn/gsa/1/{z}/{x}/{y}.png`,
                "subdomains": "123",
                "chinaCRS": "GCJ02",
                "show": true
            },
            {
                "name": "行政区划界线",
                "type": "xyz",
                "url": "https://t{s}.tianditu.gov.cn/DataServer?T=ibo_w&x={x}&y={y}&l={z}&tk=a54355999e5eb9672aa12904ea51c494",
                "subdomains": "01234567",
                "maximumLevel": 10,
                "chinaCRS": "GCJ02",
                "show": true
            }
        ]
    }
}